#!/usr/bin/env python3

import argparse
import asyncio

from .starter import starter, agent_rules, services
from .logo import print_logo


def main() -> None:
    parser = argparse.ArgumentParser(
        prog="vibe-llama",
        description="vibe-llama is a command-line tool to help you get started in the LlamIndex ecosystem with the help of vibe coding.",
    )

    subparsers = parser.add_subparsers(
        dest="command", help="Available commands", required=True
    )

    starter_parser = subparsers.add_parser(
        "starter",
        help="starter provides your coding agents with up-to-date documentation about LlamIndex, LlamaCloud Services and llama-index-workflows, so that they can build reliable and working applications! You can launch a terminal user interface by running `vibe-llama starter` or you can directly pass your agent (-a, --agent flag) and chosen service (-s, --service flag).",
    )

    starter_parser.add_argument(
        "-a",
        "--agent",
        required=False,
        help="Specify the coding agent you want to write instructions for",
        choices=[agent for agent in agent_rules],
        default=None,
    )

    starter_parser.add_argument(
        "-s",
        "--service",
        required=False,
        help="Specify the service to fetch the documentation for",
        choices=[service for service in services],
        default=None,
    )

    starter_parser.add_argument(
        "-v",
        "--verbose",
        action="store_true",
        help="Enable verbose output",
        required=False,
        default=False,
    )

    args = parser.parse_args()

    if args.command == "starter":
        print_logo()
        asyncio.run(starter(args.agent, args.service, args.verbose))

    return None



import pytest
import httpx
import os
from pathlib import Path

from src.vibe_llama.starter import starter, services
from src.vibe_llama.starter.terminal import app1, app2
from prompt_toolkit.application import Application
from src.vibe_llama.starter.utils import write_file, get_instructions


@pytest.mark.asyncio
async def test_starter() -> None:
    try:
        await starter(agent="GitHub Copilot", service="LlamaIndex", verbose=False)
        success = True
    except Exception:
        success = False
    assert success
    assert Path(".github/copilot-instructions.md").is_file()
    os.remove(".github/copilot-instructions.md")
    r = await starter(agent="GitHub Copilot")  # type: ignore
    assert r is None
    with pytest.raises(KeyError):
        await starter(agent="GitHub Copilo", service="LlamaIndex")
    with pytest.raises(KeyError):
        await starter(agent="GitHub Copilot", service="LlamaInde")


@pytest.mark.asyncio
async def test_get_instructions() -> None:
    instr = await get_instructions(services["LlamaIndex"])
    content = str(httpx.get(services["LlamaIndex"]).content, encoding="utf-8")
    assert instr is not None
    assert instr == content


def test_write_file(tmp_path: Path) -> None:
    fl = tmp_path / "hello.txt"
    write_file(str(fl), "hello world\n", "https://www.llamaindex.ai")
    assert fl.is_file()
    assert fl.stat().st_size > 0
    write_file(str(fl), "hello world", "https://www.llamaindex.ai")
    with open(fl) as f:
        content = f.read()
    assert content == "hello world\n\nhello world"


def test_terminal_apps() -> None:
    assert isinstance(app1, Application)
    assert isinstance(app2, Application)



from prompt_toolkit.shortcuts import checkboxlist_dialog
from prompt_toolkit.formatted_text import HTML
from prompt_toolkit.styles import Style
from typing import Optional, Tuple, List, cast

from .data import agent_rules, services

style = Style.from_dict(
    {
        "dialog": "bg:#F8E9D8",
        "dialog.body": "bg:#45DFF8",
        "dialog shadow": "bg:#a6a2ab",
    }
)

app1 = checkboxlist_dialog(
    title=HTML("<style fg='black'>Coding Agents</style>"),
    text="Which coding agents would you like to write instructions for?",
    values=[(agent_rules[agent], agent) for agent in agent_rules],
    style=style,
)

app2 = checkboxlist_dialog(
    title=HTML("<style fg='black'>Services</style>"),
    text="Which services would you like to get instructions for?",
    values=[(services[service], service) for service in services],
    cancel_text="Go Back",
    style=style,
)


async def run_terminal_interface() -> Optional[Tuple[List[str], List[str]]]:
    results_array1 = None
    results_array2 = None

    while not results_array2:
        results_array1 = await app1.run_async()
        if not results_array1:
            return None
        results_array2 = await app2.run_async()
    return cast(List[str], results_array1), results_array2



class InstructionsUnavailableError(Exception):
    """Raised when instructions are not available"""


class FailedToWriteFileWarning(Warning):
    """Raised when one file was not written"""


class FailedToWriteFilesError(Exception):
    """Raised when all files failed to write"""



agent_rules = {
    "GitHub Copilot": ".github/copilot-instructions.md",
    "Claude Code": "CLAUDE.md",
    "OpenAI Codex CLI": "AGENTS.md",
    "Jules": "AGENTS.md",
    "Cursor": ".cursor/rules/cursor_instructions.mdc",
    "Windsurf": ".windsurf/rules/windsurf_instructions.md",
    "Cline": ".clinerules",
    "Amp": "AGENT.md",
    "Firebase Studio": ".idx/airules.md",
    "Open Hands": ".openhands/microagents/repo.md",
    "Gemini CLI": "GEMINI.md",
    "Junie": ".junie/guidelines.md",
    "AugmentCode": ".augment/rules/augment_instructions.md",
    "Kilo Code": ".kilocode/rules/kilocode_instructions.md",
    "OpenCode": "AGENTS.md",
    "Goose": ".goosehints",
}

services = {
    "LlamaIndex": "https://docs.llamaindex.ai/en/latest/llms.txt",
    "LlamaCloud Services": "https://raw.githubusercontent.com/run-llama/vibe-llama/main/documentation/llamacloud.md",
    "llama-index-workflows": "https://raw.githubusercontent.com/run-llama/vibe-llama/main/documentation/llama-index-workflows.md",
}
